#! /bin/sh -xv
### ====================================================================
###  @UNIX-shell-file{
###     author-1        = "Van Jacobson",
###     author-2        = "Nelson H. F. Beebe",
###     version         = "3.03",
###     date            = "09 February 2005",
###     time            = "16:17:03 MST",
###     filename        = "tgrind.sh",
###     address-1       = "Lawrence Berkeley Laboratory,
###                        Berkeley, CA, USA",
###     address-2       = "University of Utah
###                        Department of Mathematics, 110 LCB
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone-2     = "+1 801 581 5254",
###     FAX-2           = "+1 801 581 4148",
###     URL-2           = "http://www.math.utah.edu/~beebe",
###     checksum        = "25182 302 1016 7538",
###     email-2         = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@computer.org (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "AmSTeX, LaTeX 2.09, LaTeX-2e, plain TeX,
###                        tgrind, typeset program listing, vgrind",
###     supported       = "yes",
###     docstring       = "This is a UNIX Bourne-shell script to produce
###			   nice program listings using TeX.
###
###			   This script was originally written in
###			   February 1985 by Van Jacobsen, based on the
###			   4.2BSD UNIX vgrind script, as a C shell
###                        script.
###
###			   This script has been substantially revised by
###			   Nelson H. F. Beebe for release 2.0 of tgrind,
###			   which contains support for new languages, new
###			   keywords in vgrindefs, and multi-column
###			   procedure name indexing.  The script was then
###			   manually translated by NHFB to a Bourne shell
###			   script, and to a standalone C program, for
###			   wider portability of tgrind version 3.00.
###
###			   The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
### ====================================================================

##########################################################################
### CHANGE THIS VALUE IF YOU UPDATE TGRIND, AND NOTIFY THE AUTHORS
### LISTED ABOVE AND IN THE MAN PAGE FILE, tgrind.man:
###
VERSION="tgrind version 3.03 [09-Feb-2005]"
###
##########################################################################
###
TFONTEDPR=/usr/local/bin/tfontedpr
LIB=/usr/local/lib/tex/inputs
TEXINPUTS=$LIB:$TEXINPUTS
export TEXINPUTS
TEX=/usr/local/bin/tex
OPTIONS=
### If your system doesn't have nawk, try gawk, or awk
AWK=nawk
BASENAME=tg_$$
FILES=
FONTFAMILY=""
FORMAT=""
HEAD=""
INCREMENT="10"
INDEX="Y"
PRINT=""
QUIET="N"
RIGHTNUMBER=""
TEXOPTIONS=

while test $# -gt 0
do
    case $1 in
    -[?])
	echo "Usage:"
	echo "        tgrind [ -? ] [ -d description-file ] [ -f ] [ -fn fontfamily ]"
	echo "                      [ -h header ] [ -i increment ] [ -l language ] [ -n ]"
	echo "                      [ -o basename ] [ -p ] [ -q ] [ -r ] [ -v ] [ -x ] [ - ]"
	echo "                      filename ..."
	exit 0
	;;

    -d)				# -d description-file
	if test $# -lt 2
	then
	    echo "tgrind: $1 option must have argument"
	    exit 1
	else
	    OPTIONS="$OPTIONS $1 $2"
	    shift
	fi
	;;
    -f)				# -f (force output to stdout)
	FORMAT="Y"
	;;
    -fn)			# -fn fontname
	if test $# -lt 2
	then
	    echo "tgrind: $1 option must have argument"
	    exit 1
	else
	    FONTFAMILY="$2"
	    shift
	fi
	;;
    -h)				# -h header
	if test $# -lt 2
	then
	    echo "tgrind: $1 option must have argument"
	    exit 1
	else
	    HEAD="$2"
	    shift
	fi
	;;
    -i)				# -i increment
	if test $# -lt 2
	then
	    echo "tgrind: $1 option must have argument"
	    exit 1
	else
	    INCREMENT="$2"
	    shift
	fi
	;;
    -l)				# -l language
	if test $# -lt 2
	then
	    echo "tgrind: $1 option must have argument"
	    exit 1
	else
	    OPTIONS="$OPTIONS $1$2" # reduce -l <language> to -l<language>
	    shift
	fi
	;;
    -n)				# -n (no bold keywords)
	OPTIONS="$OPTIONS $1"
	;;
    -o)				# -o basename
	if test $# -lt 2
	then
	    echo "tgrind: $1 option must have argument"
	    exit 1
	else
	    BASENAME="$2"
	    shift
	fi
	;;
    -p)				# -p (print PostScript to default printer)
	PRINT="Y"
	;;
    -q)				# -q (quiet)
	QUIET="Y"
	;;
    -r)				# -r (number on right)
	RIGHTNUMBER="Y"
	;;
    -v)
	echo $VERSION
	exit 0
	;;
    -x)				# -x (suppress index)
	INDEX="N"
	;;
    -)				# remaining options are files
	shift
	break
	;;
    -*)				# illegal argument
	echo "tgrind: unrecognized option: $1"
	echo "Usage:"
	echo "        tgrind [ -? ] [ -d description-file ] [ -f ] [ -fn fontfamily ]"
	echo "                      [ -h header ] [ -i increment ] [ -l language ] [ -n ]"
	echo "                      [ -o basename ] [ -p ] [ -q ] [ -r ] [ -v ] [ -x ] [ - ]"
	echo "                      filename ..."
	exit 1
	;;
    *)
	FILES="$FILES $1"
	;;
    esac
    shift
done

FILES="$FILES ""$@"		# pick up remaining arguments as file names

if test -f $BASENAME.tex -o \
	-f $BASENAME.log -o \
	-f $BASENAME.tmp -o \
	-f $BASENAME.dvi -o \
	-f $BASENAME.ps
then
    echo "Output file(s) already exist in $BASENAME.* ...terminating"
    exit 1
fi

echo "%% $VERSION for files $FILES on `date`" >$BASENAME.tex

if test "$FONTFAMILY" != ""
then
    echo "\def \TgrindFontFamily{"$FONTFAMILY"}" >>$BASENAME.tex
fi

echo "\ifx \BeginTgrind \undefined" >>$BASENAME.tex
echo "  \input tgrindmac" >>$BASENAME.tex
echo '\fi' >>$BASENAME.tex

echo "\TgrindLineIncrement = $INCREMENT" >>$BASENAME.tex

if test "$RIGHTNUMBER" = "Y"
then
    echo "\TgrindRightNumbertrue" >>$BASENAME.tex
else
    echo "\TgrindRightNumberfalse" >>$BASENAME.tex
fi

if test "$HEAD" != ""
then
    $TFONTEDPR $OPTIONS -h "$HEAD" $FILES >>$BASENAME.tex
else
    $TFONTEDPR $OPTIONS $FILES >>$BASENAME.tex
fi

### Change to directory containing output files, so temporary index
### files go there.

cd `dirname $BASENAME`
if test "$INDEX" = "Y"
then
    if test "$QUIET" = "N"
    then
	echo "Indexing..."
    fi
    $AWK -f $LIB/tgrindex.awk <$BASENAME.tex >$BASENAME.tmp && \
	/bin/mv $BASENAME.tmp $BASENAME.tex
fi

if test "$FORMAT" = "Y"
then
    cat $BASENAME.tex
    /bin/rm -f $BASENAME.tex
else
    if test "$QUIET" = "N"
    then
	echo "TeXing..."
	$TEX $TEXOPTIONS $BASENAME.tex
    else
	$TEX $TEXOPTIONS $BASENAME.tex >/dev/null
    fi


    ## CUSTOMIZATION: Put output handling commands here.

    if test -f $BASENAME.dvi
    then
	if test "$QUIET" = "N"
	then
	    echo ""
	    echo "Creating PostScript output..."
	    ## Choose ONE DVI driver here:
	    ## dvialw -q -outfile=$BASENAME.ps $BASENAME.dvi
	    dvips -o $BASENAME.ps $BASENAME.dvi
	else
	    dvips -q -o $BASENAME.ps $BASENAME.dvi
	fi
    fi

    if test "$QUIET" = "N"
    then
	echo ""
        echo "Output files..."
        ls -l $BASENAME.*
    fi

    if test "$PRINT" = "Y"
    then
	if test "$QUIET" = "N"
	then
	    echo ""
	    echo "Printing..."
        fi

	# Solaris 2.x uses lp, many other UNIX variants use lpr
	if test -f /usr/5bin/lp
	then
	    LPR=/usr/5bin/lp
	else
	    LPR=lpr
	fi
	$LPR $BASENAME.ps
    fi
fi
